// (C) Copyright 1999 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
// Created by: Jorge Lopez
//             ObjectDBX Development
//             ADF / Engineering
//             Autodesk, Inc.

/////////////////////////////////////////////
// ObjectARX defined commands

#include "StdAfx.h"
#include "StdArx.h"

// This is command 'DOCKABLE_CONTAINER'
void ac_cont_dockable_container()
{
    char cProgID[135];
    if (acedGetString(1, "ProgID of control:", cProgID) != RTNORM ||
        strlen(cProgID) == 0)
    {
        acutPrintf("Aborted command.\n");
        return;
    }

    // Get the CLSID from the ProgID. Quick sanity check to ensure
    // the control is available before getting too far. This CLSID
    // will also be used to create the control and is the key in
    // the map<>.
    USES_CONVERSION;

    CLSID clsid;
    if (FAILED(CLSIDFromProgID(A2W(cProgID), &clsid)))
    {
        acutPrintf("Error could not get CLSID from \"%s\"\n", cProgID);
        return;
    }

    // Check to see if this CLSID is currently being serviced.
    // If so, then simply re-display it and return.
    if (!gDockBars.empty())
    {
        DockBarMap::iterator iter;
        iter = gDockBars.find(clsid);

        if (iter != gDockBars.end())
        {
            CAcUiContainerBar* pDockBar = (*iter).second;
            ASSERT(pDockBar != NULL);

            CFrameWnd* pFrame = pDockBar->GetParentFrame();
            ASSERT(pFrame != NULL);

            pFrame->ShowControlBar(pDockBar, TRUE, FALSE);
            return;
        }
    }

    // Set tool id (simply the ProgId) for the docking window. 
    // Providing a tool id allows:
    // 1. Restoration of the docking window to last known location
    //    and size. This includes its docking state.
    // 2. Automatic invokation when AutoCAD starts up if the docking 
    //    window was currently visible when AutoCAD last terminated.
    CString cmdName;
    cmdName.Format("DC %s", cProgID); 

    if (cmdName.GetLength() < 128)  // sanity check to avoid crashing
        AdUiRegisterTool("accont.arx", cmdName, &clsid);

    // Create the docking container
    CAcUiContainerBar* pDockBar = NULL;
    if (pDockBar = new CAcUiContainerBar(clsid))
    {
        if (pDockBar->Create(acedGetAcadFrame(), cProgID))
        {
            // Call enable docking to provide default docking
            // mode if the OLE control did not already do so.
            if (!pDockBar->GetEnableDockingCalled())
                pDockBar->EnableDocking(CBRS_ALIGN_ANY);

            if (pDockBar->GetPreferredDockPosition() == 0)
                pDockBar->RestoreControlBar();
            else
                pDockBar->RestoreControlBar(
                    pDockBar->GetPreferredDockPosition(),
                    pDockBar->GetPreferredFloatingRect());

            // Add the new dock bar to the global list.
            gDockBars[clsid] = pDockBar;
        }
        else
        {
            acutPrintf("Error creating control \"%s\"\n", cProgID);
        }
    }
}


